# diagnostics.py

import os
import pandas as pd
import seaborn as sns
import numpy as np
import matplotlib.pyplot as plt


# --------------------------------------------------
def peaks_per_sample(df, metadata, group, path):
    """Calculate and plot the number of samples that have molecular formula."""

    # Calculating the number of samples that were assigned a molecular formula
    stats_per_sample = pd.DataFrame(df.groupby(['SampleID'])[['Mass']].size()).rename(columns={0: 'Counts'})
    stats_per_sample = stats_per_sample.reset_index()
    stats_per_sample = stats_per_sample.merge(metadata, on='SampleID')
    stats_per_sample.to_csv(os.path.join(path, 'stats_peaks_per_sample.csv'), index=False)

    sns.set_theme(style='white')
    sns.set_style('ticks')
    plt.rcParams['figure.figsize'] = (15, 5)

    # Plotting the number of molecular formulas per each of the samples
    p = sns.barplot(x="SampleID",
                    y="Counts",
                    hue=group[0],
                    data=stats_per_sample,
                    order=stats_per_sample.sort_values(by='Counts')['SampleID'],
                    saturation=0.7,
                    errcolor='.2',
                    ci="sd",
                    capsize=0.2,
                    errwidth=1.25,
                    dodge=False)

    plt.setp(p.get_xticklabels(), rotation=45, size=8, ha='right')
    plt.savefig(os.path.join(path, 'stats_peaks_per_sample.png'), dpi=300, bbox_inches="tight")
    print('The average number of peaks detected per sample is: {:.0f}'.format(
        np.mean(stats_per_sample['Counts']), ))
    plt.clf()

    return


# --------------------------------------------------
def formula_per_sample(df, metadata, group, path):
    """Calculate and plot the number of samples that have molecular formula."""

    # Calculating the number of samples that were assigned a molecular formula
    stats_per_sample = pd.DataFrame(df.groupby(['SampleID'])[['Mass']].size()).rename(columns={0: 'Counts'})
    stats_per_sample = stats_per_sample.reset_index()
    stats_per_sample = stats_per_sample.merge(metadata, on='SampleID')
    stats_per_sample.to_csv(os.path.join(path, 'stats_formula_per_sample.csv'), index=False)

    sns.set_theme(style='white')
    sns.set_style('ticks')
    plt.rcParams['figure.figsize'] = (15, 5)

    # Plotting the number of molecular formulas per each of the samples
    f = sns.barplot(x="SampleID",
                    y="Counts",
                    hue=group[0],
                    data=stats_per_sample,
                    order=stats_per_sample.sort_values(by='Counts')['SampleID'],
                    saturation=0.7,
                    errcolor='.2',
                    ci="sd",
                    capsize=0.2,
                    errwidth=1.25,
                    dodge=False)

    plt.setp(f.get_xticklabels(), rotation=45, size=8, ha='right')
    plt.savefig(os.path.join(path, 'stats_formula_per_sample.png'), dpi=300, bbox_inches="tight")
    print('The average number of masses assigned a molecular formula per sample is: {:.0f}'.format(
        np.mean(stats_per_sample['Counts']), ))
    plt.clf()

    return


# --------------------------------------------------
def error_per_group(df, group, path):
    """Calculate and plot error distribution per sample."""
    error = df.groupby([group[0], 'Mass'] if len(group) == 1
                       else [group[0], group[1], 'Mass'])['Error_ppm'].agg(['mean']).reset_index()
    error.to_csv(os.path.join(path, 'error_distribution_per_group.csv'), index=False)

    sns.set_theme(style="white")
    sns.set_style("ticks")

    grid = sns.FacetGrid(error,
                         col=group[0],
                         row=None if len(group) == 1 else group[1],
                         hue=group[0],
                         height=5)

    grid.map(plt.scatter, "Mass", "mean", s=1)
    grid.add_legend()
    grid.set_axis_labels(y_var="Error (ppm)")
    plt.savefig(os.path.join(path, 'error_distribution_per_group.png'), dpi=300, bbox_inches="tight")

    return



# transformations.py
import os
import glob
import time
import datetime
import seaborn as sns
import pandas as pd
import py4cytoscape as p4c
import numpy as np
from itertools import combinations


# ---------------------------------------------------
def get_keys(keyfile):
    """Get Biochemical Transformation Keys"""

    key = pd.read_csv(keyfile)
    key['mf'] = key['mf'].astype(float).apply(lambda x: '%.6f' % x)
    key = key.sort_values(by=['mf'])
    key_tuples = list(zip(key.Group, key.Transformation, key.Formula, key.mf))

    return key_tuples


# --------------------------------------------------
def calculate_transformations(df, keys, path):
    """Function to calculate transformations for transformation networks"""

    # Create a dataframe that has the masses of the peaks that are present in each sample, and 0 in the peaks that are
    # not in that sample
    df_transf = pd.pivot_table(df, values='NormIntensity', index=['Mass'],
                               columns=['SampleID']).reset_index()
    df_transf['Mass'] = df_transf['Mass'].astype(float).apply(lambda x: '%.6f' % x)
    df_transf.replace([0, np.nan], ['X', 'X'], inplace=True)
    for col in df_transf.columns:
        df_transf[col] = np.where((df_transf[col] != 'X'), df_transf['Mass'], df_transf[col])
    df_transf = df_transf.drop('Mass', axis=1)
    df_transf = df_transf.replace('X', 0)
    print('Calculating m/z differences per sample column can take a little while...\n')
    i = 1

    for sample in sorted(df_transf.columns):
        print(f'[{datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S %p")}] {i}\\{len(df_transf.columns)}\t{sample}')

        mz_list = set(float(x) for x in df_transf[sample] if float(x) > 0)
        print('\t\tTotal m/z values', len(mz_list))

        # make m/z substractions in all versus all fashion
        # doing all vs all the subtractions and filter
        result_tuples = [(x, y, round(abs(x - y), 6)) for x, y in combinations(mz_list, 2) if 1 < abs(x - y) < 766]

        result_tuples = [(
            r[0], r[1], r[2],
            k[0], k[1], k[2], k[3]
        ) for r in result_tuples for k in keys
            if r[2] - 0.001 <= float(k[3]) <= r[2] + 0.001]

        if len(result_tuples) == 0:
            print('No transformations were found for this sample, moving to the next one')
        
        else:
            # make np.array from list of lists
            result_tuples = np.vstack(result_tuples)

            # make pd df
            result_df = pd.DataFrame(result_tuples, columns=[
                'Feature_X', 'Feature_Y', 'Difference',
                'Group', 'Transformation', 'Formula', 'mf'])

            result_df['SampleID'] = sample

            print('   Saving results')
            filename = os.path.join(path, 'transformations_' + sample + '.csv')
            result_df.to_csv(filename, index=False)

            # Compile counts
            result_counts = pd.DataFrame(
                result_df.groupby(['SampleID', 'Group', 'Transformation', 'Formula']).size().reset_index(name='Counts'))

            total_transformations = sum(result_counts['Counts'])

            result_counts['Perc_Counts'] = result_counts['Counts'] / total_transformations
            result_counts = result_counts.sort_values(by="Counts")

            # Save final_counts
            filename = os.path.join(path, 'counts_' + sample + '.csv')
            result_counts.to_csv(filename, index=False)
        i = i + 1

    print("\u2713 Done!")
    return


# --------------------------------------------------
def summarize_transformations(path):
    """Create a table summarizing the number of transformations of each sample"""

    files_path = os.path.join(path, 'transf_by_sample', 'counts_*.csv')
    files = glob.glob(files_path)
    summary_counts = pd.DataFrame()

    for file in files:
        df = pd.read_csv(file)
        summary_counts = pd.concat([summary_counts, df], axis=0)

    filename = os.path.join(path, 'Transformations_summary_counts.csv')
    summary_counts.to_csv(filename, index=False)

    files_path = os.path.join(path, 'transf_by_sample', 'transformations_*.csv')
    files = glob.glob(files_path)
    summary_transf = pd.DataFrame()

    for file in files:
        df = pd.read_csv(file)
        summary_transf = pd.concat([summary_transf, df], axis=0)

    filename = os.path.join(path, 'Transformations_summary_all.csv')
    summary_transf.to_csv(filename, index=False)
    return


# --------------------------------------------------
def get_node_table(df, path):
    """Create a node table for the transformation networks"""

    node_table = df[['Mass', 'C', 'H', 'O', 'N', 'S', 'P', 'OC', 'HC', 'NOSC',
                     'GFE', 'Class', 'MolecularFormula', 'El_comp']].drop_duplicates('Mass')

    filename = os.path.join(path, 'node_table.csv')
    node_table.to_csv(filename, index=False)

    return node_table


# --------------------------------------------------
def create_cytoscape_network(node_table, path):
    """Create a cytoscape network using the node table"""

    node_table['Mass'] = round(node_table['Mass'], 4)
    node_table['Mass'] = node_table['Mass'].astype(str)
    node_table = node_table.rename(columns={'Mass': 'id'})

    # Create a vector of colors for the compound classes
    mol_classes = list(np.unique(node_table['Class']))
    node_colors = sns.color_palette('Set3', len(mol_classes)).as_hex()

    network_stats = []

    # Create a list of the transformation files to be used as the edge tables for the networks
    files_path = os.path.join(path, 'transf_by_sample', 'transformations_*.csv')
    edge_files = glob.glob(files_path)
    i = 1

    for file in edge_files:
        edge_table = pd.read_csv(file)
        edge_table['Feature_X'] = round(edge_table['Feature_X'], 4)
        edge_table['Feature_X'] = edge_table['Feature_X'].astype(str)
        edge_table['Feature_Y'] = round(edge_table['Feature_Y'], 4)
        edge_table['Feature_Y'] = edge_table['Feature_Y'].astype(str)
        # Renaming columns for clarity
        edge_table = edge_table.rename(columns={'Feature_X': 'source', 'Feature_Y': 'target'})
        print(f'[{datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S %p")}] {i}\\{len(edge_files)}'
              f'\tCreating network for sample: {edge_table.SampleID.unique()[0]}')
        with pd.option_context('mode.chained_assignment', None):
            p4c.create_network_from_data_frames(None, edge_table,
                                                source_id_list='source',
                                                target_id_list='target',
                                                interaction_type_list='Transformation',
                                                title=edge_table.SampleID.unique()[0],
                                                collection='Transformation networks')
            # Load node table with data information
            p4c.load_table_data(node_table, data_key_column='id')

        # Create a vector of colors for the transformation groups
        transformation_group = list(np.unique(edge_table['Group']))
        edge_colors = sns.color_palette('dark', len(transformation_group)).as_hex()

        # Coloring the table based on compound classes and transformation groups
        p4c.set_node_shape_default('ELLIPSE')
        p4c.set_node_color_mapping('Class', mol_classes, node_colors, mapping_type='d')
        p4c.set_edge_color_mapping('Group', transformation_group, edge_colors, mapping_type='d')

        # Run the Network Analyzer command for cytoscape and capture its output in a list of lists
        network_analyzer = p4c.commands_post('analyzer analyze')
        network_stats.append(
            {'SampleID': edge_table.SampleID.unique()[0],
             'netHE': network_analyzer['heterogeneity'],
             'nNodes': network_analyzer['nodeCount'],
             'nEdges': network_analyzer['edgeCount'],
             'meanNeighbors': network_analyzer['avNeighbors'],
             'clusterCoeff': network_analyzer['cc'],
             'netDensity': network_analyzer['density'],
             'netCentralization': network_analyzer['centralization'],
             'connectComponents': network_analyzer['ncc'],
             'pathLength': network_analyzer['avSpl'],
             'netDiameter': network_analyzer['diameter'],
             'netRadius': network_analyzer['radius']}
        )

        time.sleep(5)
        i = i + 1

    network_stats = pd.DataFrame(network_stats)
    filename = os.path.join(path, 'network_summary_statistics.csv')
    network_stats.to_csv(filename, index=False)

    return



# exploration.py

import os
import subprocess
from shutil import which
from platform import system


# --------------------------------------------------
def write_r_script(rscript, outdir, metadata_file, groups, norm_method='max'):
    """Write R scripts based on the provided Rscripts templates"""

    # Fixing the paths obtained in Windows, so they can work well in R and in Python
    current_dir = os.getcwd().replace('\\', '/')
    metadata_file = metadata_file.replace('\\', '/')
    metabo_home = os.path.split(os.path.realpath(__file__))[0].replace('\\', '/')

    # Modify the specified R script template with the desired values
    r_in = os.path.join(os.path.split(os.path.realpath(__file__))[0], 'R_scripts_templates', rscript)
    r_in = open(r_in)
    r_file = os.path.join(outdir, rscript.replace('_template', ''))
    r_fh = open(r_file, 'w')
    for line in r_in:
        r_fh.write(line.replace('%currentdir%', current_dir)
                   .replace('%metadata%', metadata_file)
                   .replace('%Metabo_HOME%', metabo_home)
                   .replace('%outdir%', os.path.split(outdir)[0])
                   .replace('%group1%', groups[0])
                   .replace('%group2%', groups[1] if len(groups) == 2 else 'NULL')
                   .replace('%norm_method%', norm_method)
                   )
    r_fh.close()

    return r_file


# --------------------------------------------------
def run_r(rscript):
    """Execute R scripts"""

    # Check if the script is being run in Windows, or Linux and Mac so the correct Rscript executable is chosen
    rscript_exe = 'Rscript.exe' if system() == 'Windows' else 'Rscript'

    # Execute the R script
    if which(rscript_exe) is not None:
        r_cmd = [rscript_exe, rscript]
        p = subprocess.Popen(
            r_cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        out, err = p.communicate()
        # if out:
        #     print(out)

        # if err:
        #     print(err)
    else:
        print('Rscript not accesible')



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# Required package

REQUIRED = ['pandas', 'numpy', 'py4cytoscape',
            'seaborn', 'more-itertools', 'argparse']

# This call to setup() does all the work
setup(
    name="metabodirect",
    version="0.2.11",
    description="Analyze FT-ICR-MS data",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/Coayala/MetaboDirect",
    author="Christian Ayala",
    author_email="cayalaortiz@email.arizona.edu",
    license="LICENSE",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=["metabodirect"],
    include_package_data=True,
    install_requires=REQUIRED,
    entry_points={
        "console_scripts": [
            "metabodirect=metabodirect.__main__:main",
            "test_normalization=metabodirect.test_normalization:main",
            "create_networks=metabodirect.create_networks:main"
        ]
    },
)



# get_args.py

import os
import pandas as pd
import numpy as np
from statsmodels.stats.weightstats import DescrStatsW


# --------------------------------------------------------------
def filter_c13(df):
    """ Filter predicted formulas with 13C.
    Returns filtered df and n excluded """

    shape_i = df.shape[0]

    df = df[df['C13'] == 0]
    df = df.reset_index(drop=True)

    shape_f = df.shape[0]

    n_excluded = shape_i - shape_f

    return df, n_excluded


# --------------------------------------------------------------
def filter_mz(df, min_mz=None, max_mz=None):
    """ Optional filtering. Filter between a range of m/z """

    df = df[df['Mass'].between(min_mz, max_mz)]
    df = df.reset_index(drop=True)

    return df


# --------------------------------------------------------------
def filter_error_ppm(df, err_range=0.5):
    """ Filter based on error range of - err_range to + err_range """

    df = df[df['Error_ppm'].between(-err_range, err_range)]

    return df


# --------------------------------------------------------------
def calculate_ratios(df):
    """ Calculate ratios of O:C and H:C and indices of NOSC, GFE, DBE, AI, et al. """

    df[['C', 'H', 'O', 'N', 'S', 'P']] = df[['C', 'H', 'O', 'N', 'S',
                                             'P']].replace(np.nan, 0)

    df[['C', 'H', 'O', 'N', 'S', 'P']] = df[['C', 'H', 'O', 'N', 'S',
                                             'P']].astype(float)

    df['OC'] = df['O'] / df['C']
    df['HC'] = df['H'] / df['C']

    # Thermodynamics
    df['NOSC'] = -((4 * df['C'] + df['H'] - 3 * df['N'] - 2 * df['O'] +
                    5 * df['P'] - 2 * df['S']) / (df['C'])) + 4

    df['GFE'] = -(28.5 * df['NOSC']) + 60.3

    # Koch, B. P. and Dittmar, T. doi:10.1002/rcm.2386, 2006.
    df['DBE'] = 1 + 0.5 * (2 * df['C'] - df['H'] + df['N'] + df['P'])

    df['DBE_O'] = (1 + 0.5 *
                   (2 * df['C'] - df['H'] + df['N'] + df['P'])) - df['O']

    df['AI'] = (1 + df['C'] - df['O'] - df['S'] -
                ((df['H'] + df['P'] + df['N']) * 0.5)) / (
                       df['C'] - df['O'] - df['S'] - df['N'] - df['P'])

    df['AI_mod'] = (1 + df['C'] - (df['O'] * 0.5) - df['S'] - (
            (df['H'] + df['P'] + df['N']) * 0.5)) / (df['C'] - (df['O'] * 0.5) -
                                                     df['S'] - df['N'] - df['P'])

    df['DBE_AI'] = 1 + df['C'] - df['O'] - df['S'] - (
            0.5 * (df['H'] + df['N'] + df['P']))

    list_new_columns = [
        'OC', 'HC', 'NOSC', 'GFE', 'DBE', 'DBE_O', 'AI', 'AI_mod', 'DBE_AI'
    ]

    df[list_new_columns] = df[list_new_columns].replace([-np.inf, +np.inf],
                                                        [0, 0])

    list_dbes = ['DBE', 'DBE_O', 'DBE_AI']

    df[list_dbes] = df[list_dbes].replace(1, np.nan)

    df = molecular_formula(df)

    return df


# --------------------------------------------------------------
def calculate_classes(df):
    """ Calculate compound classes based on M. Tfaily boundaries """

    boundaries = [
        (df['OC'].between(0, 0.3)) & (df['HC'].between(1.5, 2.5)),
        (df['OC'].between(0, 0.125)) & (df['HC'].between(0.8, 1.5)),
        (df['OC'].between(0, 0.95)) & (df['HC'].between(0.2, 0.8)),
        (df['OC'].between(0.3, 0.55)) & (df['HC'].between(1.5, 2.3)),
        (df['OC'].between(0.55, 0.7)) & (df['HC'].between(1.5, 2.2)),
        (df['OC'].between(0.7, 1.5)) & (df['HC'].between(1.5, 2.5)),
        (df['OC'].between(0.125, 0.65)) & (df['HC'].between(0.8, 1.5)),
        (df['OC'].between(0.65, 1.1)) & (df['HC'].between(0.8, 1.5))
    ]

    choices = [
        'Lipid', 'Unsaturated hydrocarbon', 'Condensed hydrocarbon', 'Protein',
        'Amino sugar', 'Carbohydrate', 'Lignin', 'Tannin'
    ]

    df['Class'] = np.select(boundaries, choices, default='Other')

    df = reorder_columns(df)

    return df


# --------------------------------------------------------------
def get_list_samples(df):
    """ Returns a list of sample names and a list of Formularity columns """

    from_formularity = [
        'Mass', 'C', 'H', 'O', 'N', 'C13', 'S', 'P', 'Na', 'El_comp', 'Class',
        'NeutralMass', 'Error_ppm', 'Candidates'
    ]

    calculated_indices = [
        'MolecularFormula', 'OC', 'HC', 'NOSC', 'GFE', 'DBE', 'DBE_O', 'AI', 'AI_mod', 'DBE_AI'
    ]

    from_formularity.extend(calculated_indices)

    extended_list = from_formularity

    list_samples = [x for x in df.columns.to_list() if x not in extended_list]

    return list_samples


# --------------------------------------------------------------
def reorder_columns(df):
    """ Reorder columns so indices are not in the end of the """

    from_formularity = [
        'Mass', 'C', 'H', 'O', 'N', 'C13', 'S', 'P', 'Na', 'El_comp', 'Class',
        'NeutralMass', 'Error_ppm', 'Candidates'
    ]

    calculated_indices = [
        'MolecularFormula', 'OC', 'HC', 'NOSC', 'GFE', 'DBE', 'DBE_O', 'AI', 'AI_mod', 'DBE_AI'
    ]

    from_formularity.extend(calculated_indices)

    samples_list = get_list_samples(df)

    correct_order = from_formularity

    correct_order.extend(samples_list)

    df = df[correct_order]

    return df


# --------------------------------------------------------------
def molecular_formula(df):
    """ Get a molecular formula """

    elements = df[['C', 'H', 'O', 'N', 'S', 'P']]
    elements = elements.applymap(int)
    elements = elements.applymap(str)

    for col in elements.columns.to_list():
        elements[col] = col + elements[col]

    for col in elements.columns.to_list():
        elements[col] = elements[col].replace(f"{col}1", col)
        elements[col] = elements[col].replace(f"{col}0", '')

    df['MolecularFormula'] = elements['C'] + elements['H'] + elements[
        'O'] + elements['N'] + elements['S'] + elements['P']

    return df


# --------------------------------------------------------------
def get_summary(df, on='Class'):
    """ Get summary of class or elemental composition """

    samples = get_list_samples(df)

    samples.append(on)

    t = df[samples]

    t = t.melt(id_vars=[on], var_name='SampleID', value_name='NormIntensity')

    t = t[t['NormIntensity'] != 0].reset_index(drop=True)

    comp = t.groupby(['SampleID', on
                      ]).size().reset_index().rename(columns={0: 'Count'})

    comp = pd.pivot_table(comp, index='SampleID', values='Count', columns=on)

    comp_p = round(comp.div(comp.sum(axis=1), axis=0) * 100, 2).reset_index()

    return comp_p


# --------------------------------------------------------------
def get_summary_indices(df, on='NOSC'):
    """ Get the summary stats for the indices: median, mean, std, weighted mean and weighted std """

    samples = get_list_samples(df)

    samples.append(on)

    t = df[samples]

    t = t.melt(id_vars=[on], var_name='SampleID', value_name='NormIntensity')

    t = t[t['NormIntensity'] > 0].reset_index(drop=True)

    t_agg = t.groupby(['SampleID']).agg({on: ['median', 'mean', 'std']})

    t_agg.columns = t_agg.columns.map('_'.join)

    t_agg = t_agg.reset_index()

    t_agg[[on + '_w_mean', on + '_w_std']] = ''

    for sample in t['SampleID'].unique():
        # print(sample)
        temp = t[t['SampleID'] == sample]
        wdf = DescrStatsW(temp[on], weights=temp['NormIntensity'])
        t_agg.loc[t_agg['SampleID'] == sample, on + '_w_mean'] = wdf.mean
        t_agg.loc[t_agg['SampleID'] == sample, on + '_w_std'] = wdf.std

    return t_agg


# --------------------------------------------------------------
def get_matrix(df):
    """ Make a matrix for multivariate analysis like NMDS, PCA.
    Matrix based on m/z and not molec formula """

    samples = get_list_samples(df)

    samples.append('Mass')

    t = df[samples].set_index('Mass').rename(columns={'index': 'SampleID'})

    t = t.replace(0, np.nan)

    n_samples = np.ceil((len(samples) - 1) / 5)

    # print(t.shape)

    t = t.dropna(axis=0, thresh=n_samples)

    # print(t.shape)

    t = t.replace(np.nan, 0)

    matrix = t.copy()

    return matrix


# --------------------------------------------------
def make_directories(outdir):
    """Create and return a list of directories for the outputs of each step of the pipeline."""
    project_name = outdir
    preprocess_dir = '1_preprocessing_output'
    diagnostics_dir = '2_diagnostics'
    exploratory_dir = '3_exploratory'
    diversity_dir = '4_chemodiversity'
    stats_dir = '5_statistics'
    transf_dir = '6_transformations'
    transf_sub_dir = os.path.join(transf_dir, 'transf_by_sample')

    list_dir = [
        preprocess_dir, diagnostics_dir, exploratory_dir, diversity_dir, stats_dir, transf_dir, transf_sub_dir
    ]

    list_dir = [os.path.join(project_name, x) for x in list_dir]

    if not os.path.exists(project_name):
        os.makedirs(project_name)

    for d in list_dir:
        if not os.path.exists(d):
            os.makedirs(d)

    return list_dir


# --------------------------------------------------
def sample_filtering(df, metadata, filter_by, path):
    """Filter samples based on selected features and values."""

    # Get the variable a values specified for sample filtering
    filter_col = filter_by[0]
    filter_values = filter_by[1].split(sep=',')

    # Saving a new metadata file containing only the samples remaining after filtering
    filt_metadata = pd.DataFrame()
    for i in filter_values:
        filt_metadata = filt_metadata.append(metadata[metadata[filter_col] == i])
    filt_metadata.to_csv(os.path.join(path, 'filtered_metadata.csv'), index=False)

    # Saving a new input file containing only the samples remaining after filtering
    from_formularity = [
        'Mass', 'C', 'H', 'O', 'N', 'C13', 'S', 'P', 'Na', 'El_comp', 'Class',
        'NeutralMass', 'Error_ppm', 'Candidates'
    ]
    col_df = filt_metadata['SampleID'].to_list()
    from_formularity.extend(col_df)
    filt_df = df[from_formularity]
    filt_df.to_csv(os.path.join(path, 'filtered_input.csv'), index=False)

    return filt_df, filt_metadata


# --------------------------------------------------
def data_filtering(df, filter_values):
    """Filter data based on a specified m/z range, presence of isotopes and quality."""

    print(f'Number of m/z in provided file: {df.shape[0]}')
    # Filtering masses determined to be isotopic
    filt_df, n_excluded = filter_c13(df)
    print(f'Number of masses excluded because of C13 isotope: {n_excluded}')
    print(f'Number of m/z remaining after isotope-filtering: {filt_df.shape[0]}')

    # Filter peaks based in m/z range
    filt_df = filter_mz(filt_df, filter_values[0], filter_values[1]) if filter_values else filt_df
    print(f'Number of m/z remaining after m/z-filtering: {filt_df.shape[0]}')

    # Filter samples with error higher than 0.5 ppm
    filt_df = filter_error_ppm(filt_df, err_range=0.5)
    print(f'Number of m/z after error filtering (0.5 ppm): {filt_df.shape[0]}')

    return filt_df


# --------------------------------------------------
def thermo_idx_and_classes(df
                           ):
    """Calculate thermodynamic indices and the associated class of compounds based on the molecular formula."""

    df = calculate_ratios(df)
    df = calculate_classes(df)

    return df


# --------------------------------------------------
def data_normalization(df, norm_method, norm_subset, subset_parameter=1, log=False):
    """Normalize direct injection data based on the specified parameters."""

    samples = get_list_samples(df)
    input_data = df[samples]

    # Pivoting the data, so it removes the masses that are not present in any of the samples
    input_data.insert(0, 'Mass', list(df['Mass']))
    input_data = pd.melt(input_data, id_vars='Mass', value_vars=samples, var_name='SampleID', value_name='Intensity')
    input_data = input_data[input_data['Intensity'] > 0]
    input_data = input_data.pivot(index='Mass', columns='SampleID', values='Intensity')
    npeaks, nsamples = np.shape(input_data)

    if log:
        assert 0 not in input_data, 'Cannot calculate log 0, please consider changing this values to 1'
        input_data = np.log(input_data)

    # Perform sample subset to calculate normalization factors
    if norm_subset == 'PPP':
        min_observations = int(subset_parameter * nsamples)
        selected_peaks = []
        for peak in range(0, npeaks):
            if sum(input_data.iloc[peak, :] > 0) > min_observations:
                keep = input_data.iloc[peak, :].name
                selected_peaks.append(keep)
    elif norm_subset == 'LOS':
        ntop_peaks = int(subset_parameter * npeaks)
        keep = []
        for sample in range(0, nsamples):
            ord_peaks = input_data.iloc[:, sample].sort_values(ascending=False)
            keep.extend(list(ord_peaks.iloc[0:ntop_peaks, ].index))
        selected_peaks = list(np.unique(keep))
    else:
        selected_peaks = list(input_data.index)

    # Calculate normalization factors
    sample_mean = input_data.loc[selected_peaks, :].mean(axis=0, skipna=True)
    sample_min = input_data.loc[selected_peaks, :].min(axis=0, skipna=True)
    sample_max = input_data.loc[selected_peaks, :].max(axis=0, skipna=True)
    sample_median = input_data.loc[selected_peaks, :].median(axis=0, skipna=True)
    sample_std = input_data.loc[selected_peaks, :].std(axis=0, skipna=True)
    sample_sum = input_data.loc[selected_peaks, :].sum(axis=0, skipna=True)

    # Normalize data based on chosen method
    if norm_method == 'mean':
        norm_data = (input_data-sample_mean) / (sample_max-sample_min)
    elif norm_method == 'median':
        norm_data = (input_data-sample_median) / (sample_max-sample_min)
    elif norm_method == 'zscore':
        norm_data = (input_data-sample_mean) / sample_std
    elif norm_method == 'sum':
        norm_data = input_data / sample_sum
    elif norm_method == 'max':
        norm_data = input_data / sample_max
    elif norm_method == 'minmax':
        norm_data = (input_data-sample_min) / (sample_max-sample_min)
    elif norm_method == 'binary':
        norm_data = input_data.copy()
        norm_data[norm_data > 0] = 1
    else:
        norm_data = input_data.copy()

    norm_data['Mass'] = norm_data.index
    norm_data = norm_data.reset_index(drop=True)
    norm_data = norm_data.replace(np.nan, 0)
    temp = df[df.columns[~df.columns.isin(samples)]]
    norm_data = temp.merge(norm_data, on='Mass')

    # Non-normalized data for chemodiversity analysis
    input_data['Mass'] = input_data.index
    nonorm_data = input_data.reset_index(drop=True)
    nonorm_data = nonorm_data.replace(np.nan, 0)
    nonorm_data = temp.merge(nonorm_data, on='Mass')

    return norm_data, nonorm_data


# --------------------------------------------------
def calculate_summaries(df, path):
    """Get summaries for class composition, elemental composition and thermodynamic indices."""

    class_comp = get_summary(df, on='Class')
    el_comp = get_summary(df, on='El_comp')

    idx_stats = pd.DataFrame(get_list_samples(df), columns=['SampleID'])
    for i in ['NOSC', 'GFE', 'DBE', 'AI']:
        temp = get_summary_indices(df, i)
        idx_stats = idx_stats.merge(temp, on='SampleID')

    class_comp.to_csv(os.path.join(path, 'class_composition.csv'), index=False)
    el_comp.to_csv(os.path.join(path, 'elemental_composition.csv'), index=False)
    idx_stats.to_csv(os.path.join(path, 'indices_statistics.csv'), index=False)

    return



#!/usr/bin/env python3
"""
Author : Christian Ayala <cayalaortiz@email.arizona.edu>
Date   : 2023-01-18
Purpose: Program to run MetaboDirect scripts
"""

import argparse
import os
import sys
import pandas as pd
import py4cytoscape as p4c

from metabodirect import preprocessing, r_control, transformations


# --------------------------------------------------
def get_args():
    """Get command-line arguments"""

    parser = argparse.ArgumentParser(
        description='Program for creating molecular transformation networks, '
                    'based on previously calculated transformations',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('outdir',
                        help='Output directory used to create networks with metabodirect and the -t option',
                        metavar='OUTDIR',
                        type=str)

    parser.add_argument('metadata_file',
                        help='Metadata file used in the analysis, if a filtered metadata was generated please enter '
                             'that one',
                        metavar='METADATA',
                        type=str)

    parser.add_argument('group',
                        help='Grouping variables for coloring and faceting figures (Max 2)',
                        metavar='GROUP',
                        type=str,
                        nargs='+')

    args = parser.parse_args()

    if len(args.group) > 2:
        parser.error('Incorrect number of variables for grouping. Please enter exactly two values')

    return args


# --------------------------------------------------
def main():
    """Create networks with Cytoscape"""

    args = get_args()
    list_dir = preprocessing.make_directories(args.outdir)
    node_table = pd.read_csv(os.path.join(list_dir[5], 'node_table.csv'))

    check = ''
    while check != 'You are connected to Cytoscape!':
        cytoscape = input(f'Please open Cytoscape and press the ENTER key [q for quit].')
        if cytoscape == '':
            try:
                check = p4c.cytoscape_ping()
            except (ValueError, Exception):
                print('Cytoscape is not open.')
        elif cytoscape == 'q':
            sys.exit()
        else:
            check = 'Cytoscape not open'

    print('Starting network construction on Cytoscape')

    transformations.create_cytoscape_network(node_table, path=list_dir[5])

    network_stats_script = r_control.write_r_script('network_stats_template.R', outdir=list_dir[5],
                                                    metadata_file=args.metadata_file,
                                                    groups=args.group, norm_method='max')

    print(f'Running R script: {network_stats_script}')
    r_control.run_r(network_stats_script)
    print(f'Find results and R script in the directory: {os.path.abspath(list_dir[5])}')

    print('------------------------\nNetwork creation and analysis finished\n------------------------\n')


# --------------------------------------------------
if __name__ == '__main__':
    main()
